local pbxutils = require"core.pbxutils"
local api_sync = {}


----------------------------------------------------------------
-- Implementation details
----------------------------------------------------------------

local function __fix_sound_path(path, host)
    if not path or path == "" then
        return ""
    end

    if not host or host == "" then
        return path
    end

    local default_sounds_file = "/var/lib/wms/exclude_custom_sounds.txt"
    local sound_files = {}
    if require"core.fsutils".is_file(default_sounds_file) then
        local fp = io.open(default_sounds_file)
        if fp then
            local content = fp:read("*a")
            io.close (fp)
            if content then
                for file in string.gmatch(content, "[^\r\n]+") do
                    table.insert(sound_files, require"core.pbxutils".split(file, "%.")[1])
                end
            end
        end
    end

    local isCustomSound = true
    for _, cf in pairs(sound_files) do
        if string.find(path, cf, 1, true) then
            isCustomSound = false
            break
        end
    end

    if isCustomSound then
        path = string.format("%s/%s", host, path)
    end
    return path
end


local fix_app = {
     Goto = function(params, options)
        if pbxutils.in_table(params.dialplan, options.contexts) then
            params.dialplan = string.format("%s_%s", options.host, params.dialplan)
        end
        return params
    end
    ,GotoIf = function(params, options)
        if pbxutils.in_table(params.dialplan, options.contexts) then
            params.dialplan = string.format("%s_%s", options.host, params.dialplan)
        end
        return params
    end
    ,Hangup = function(params, options)
        return params
    end
    ,Playback = function(params, options)
        params.filename = __fix_sound_path(params.filename, options.host)
        return params
    end
    ,Set = function(params, options)
        return params
    end
    ,Queue = function(params, options)
        if params.queueId and params.queueId ~= "" then
            params.queueId = options.prefix + tonumber(params.queueId)
        end
        if params.msgForOperator and params.msgForOperator ~= "" then
            params.msgForOperator = __fix_sound_path(params.msgForOperator, options.host)
        end
        return params
    end
    ,CustomApp = function(params, options)
        if string.find(params.cwApp, "QUEUE_MEMBER") then
            params.cwApp = "Noop(Function QUEUE_MEMBER is not supported on failover)"
        end
        return params
    end
    ,Timetable = function(params, options)
        if params.timetableId and params.timetableId ~= "" then
            params.timetableId = options.prefix + tonumber(params.timetableId)
        end
        if pbxutils.in_table(params.dialplan, options.contexts) then
            params.dialplan = string.format("%s_%s", options.host, params.dialplan)
        end
        return params
    end
    ,DialPhone = function(params, options)
        params.msgForOperator = __fix_sound_path(params.msgForOperator, options.host)
        return params
    end
    ,GotoIfGroupCount = function(params, options)
        params.groupname = ""
        if pbxutils.in_table(params.dialplan, options.contexts) then
            params.dialplan = string.format("%s_%s", options.host, params.dialplan)
        end
        return params
    end
    ,VoiceMail = function(params, options)
        return params
    end
    ,VerifyCaller = function(params, options)
        if pbxutils.in_table(params.dialplan, options.contexts) then
            params.dialplan = string.format("%s_%s", options.host, params.dialplan)
        end
        return params
    end
    ,Answer = function(params, options)
        return params
    end
    ,Background = function(params, options)
        params.filename = __fix_sound_path(params.filename, options.host)
        return params
    end
    ,GotoIfSwitchIsOn = function(params, options)
        if params.switchId and params.switchId ~= "" then
            params.switchId = options.prefix + tonumber(params.switchId)
        end
        if pbxutils.in_table(params.dialplan, options.contexts) then
            params.dialplan = string.format("%s_%s", options.host, params.dialplan)
        end
        return params
    end
    ,WaitExten = function(params, options)
        return params
    end
    ,GotoIf3SwitchIsOnOrBlinking = function(params, options)
        if params.switchId and params.switchId ~= "" then
            params.switchId = options.prefix + tonumber(params.switchId)
        end
        if pbxutils.in_table(params.dialplan, options.contexts) then
            params.dialplanOn = string.format("%s_%s", options.host, params.dialplanOn)
        end
        if pbxutils.in_table(params.dialplan, options.contexts) then
            params.dialplanBlinking = string.format("%s_%s", options.host, params.dialplanBlinking)
        end
        return params
    end
    ,DialRemotePbx = function(params, options)
        return params
    end
}


local function __validateConfig(host, task, state)
    if state == "prepare" then
        if not host or host == "" then
            return false, "empty host"
        end

        if not (task.config and task.config.srcpbx) then
            return false, "configuration -> dsthost is missing"
        end

        if not task.config.srcpbx.apiuser then
            return false, "api user is not specified"
        end

        if not task.config.srcpbx.apipass then
            return false, "api password is not specified"
        end

        if type(task.config.srcpbx.proceduresForCopy) ~= "table" then
            return false, "proceduresForCopy variable is not a table"
        end

        if type(task.config.srcpbx.proceduresForMerge) ~= "table" then
            return false, "proceduresForMerge variable is not a table"
        end
    elseif state == "enroll" then
        if not (task.config and task.config.dstpbx) then
            return false, "configuration -> dsthost is missing"
        end

        if not task.config.dstpbx.apiuser then
            return false, "api user is not specified"
        end

        if not task.config.dstpbx.apipass then
            return false, "api password is not specified"
        end

        if not (task.modules.api_sync and task.modules.api_sync.data) then
            return false, "data from prepare function not found"
        end

        if type(task.config.srcpbx.proceduresForMerge) ~= "table" then
            return false, "proceduresForMerge variable is not a table"
        end
    else
        return false, "requsted validation for unknown state"
    end

    return true
end


----------------------------------------------------------------
-- Public interface
----------------------------------------------------------------
function api_sync.prepare(host, task)
    local result, err = __validateConfig(host, task, "prepare")
    if not result then
        return false, err
    end

    local prefix, err = require"libs.idrange".get_range(host)
    if not prefix then
        return false, err
    end

    local api, err = require"libs.dialplanapi":new(task.config.srcpbx.apiuser, task.config.srcpbx.apipass, task.config.srcpbx.apihost,
                                                    task.config.srcpbx.apiport, task.config.srcpbx.apiproto, task.config.srcpbx.apiverison)
    if err then
        return false, err
    end

    task.modules.api_sync = {}
    task.modules.api_sync.data = {}

    local all_procedures = {}
    for _, v in ipairs(task.config.srcpbx.proceduresForMerge) do table.insert(all_procedures, v) end
    for _, v in ipairs(task.config.srcpbx.proceduresForCopy) do table.insert(all_procedures, v) end

    for _, context in ipairs(all_procedures) do

        local data, err = api:getDialplanByName(context)

        if not err then
            local rv = {}
            rv.description = data.description
            rv.numbers = {}
            rv.includedDialplans = {}
            if pbxutils.in_table(context, task.config.srcpbx.proceduresForCopy) then
                rv.name = string.format("%s_%s", host, data.name)
            else
                rv.name = data.name
            end

            local options = {host = host, prefix = prefix, contexts = task.config.srcpbx.proceduresForCopy}

            for _, rec in ipairs(data.numbers) do
                local apps = {}
                for _, app in ipairs(rec.apps) do
                    if fix_app[app.name] then
                        table.insert(apps, {name = app.name, params = fix_app[app.name](app.params, options)})
                    elseif not task.config.srcpbx.removeUnsupportedApps then
                        table.insert(apps,  {name = app.name, params = app.params})
                    end
                end
                if #apps ~= 0 then
                    table.insert(rv.numbers, {number = rec.number, apps = apps})
                end
            end

            task.modules.api_sync.data[context] = rv

        elseif err and err ~= "context not found" then
            return false, err
        end
    end

    return true
end


function api_sync.enroll(host, task)
    local result, err = __validateConfig(host, task, "enroll")
    if not result then
        return false, err
    end

    local api, err = require"libs.dialplanapi":new(task.config.dstpbx.apiuser, task.config.dstpbx.apipass, task.config.dstpbx.apihost,
                                                   task.config.dstpbx.apiport, task.config.dstpbx.apiproto, task.config.dstpbx.apiverison)
    if err then
        return false, err
    end

    for context, params in pairs(task.modules.api_sync.data) do
        if pbxutils.in_table(context, task.config.srcpbx.proceduresForCopy) then
            local rv, err = api:addOrUpdateDialplanByName(params)
            if err then
                return false, err
            end
        else
            local rv, err = api:mergeDialplansByName(params)
            if err then
                return false, err
            end
        end
    end

    return true
end


function api_sync.cleanup(host, task)
    return true
end


return api_sync
